@@ -1,4 +1,3 @@
-use std::env;
 use std::ffi::OsString;
 use std::cmp::Ordering;
 use std::process::exit;
@@ -262,6 +261,9 @@ fn main() {
 }
 
 fn real_main() -> Result<()> {
+    // If there were any errors reading the input.
+    let mut bad_input = false;
+
     // Number of certs that have issues.
     let mut certs_with_issues = 0;
     // Whether we were unable to fix at least one issue.
@@ -316,11 +318,24 @@ fn real_main() -> Result<()> {
         .kind(armor::Kind::SecretKey)
         .build()?;
 
-    for input in inputs {
+    'next_input: for input in inputs {
         let filename = if input == "-" { "/dev/stdin".into() } else { input };
-        let certp = CertParser::from_file(filename)?;
-        'next_cert: for certo in certp {
+        let certp = CertParser::from_file(&filename)?;
+        'next_cert: for (certi, certo) in certp.enumerate() {
             match certo {
+                Err(err) => {
+                    if ! args.quiet {
+                        if certi == 0 {
+                            eprintln!("{:?} does not appear to be a keyring: {}",
+                                      filename, err);
+                        } else {
+                            eprintln!("Encountered an error parsing {:?}: {}",
+                                      filename, err);
+                        }
+                    }
+                    bad_input = true;
+                    continue 'next_input;
+                }
                 Ok(cert) => {
                     // Whether we found at least one issue.
                     let mut found_issue = false;
@@ -766,8 +781,6 @@ fn real_main() -> Result<()> {
 
                     next_cert!();
                 }
-                Err(err) => panic!("parsing cert #{}: {}",
-                                   certs_valid + certs_invalid, err),
             }
         }
     }
@@ -796,19 +809,12 @@ fn real_main() -> Result<()> {
         }};
     }
 
-    if ! args.quiet {
+    if certs_with_issues > 0 {
         eprintln!("Examined {} {}.",
                   certs_valid + certs_invalid,
                   pl(certs_valid + certs_invalid,
                      "certificate", "certificates"));
-    }
-    if certs_with_issues == 0 {
-        if ! args.quiet {
-            eprintln!("No issues found (see `{} --help` for a list of issues \
-                       that are checked for).",
-                      env::args().nth(0).unwrap_or("sq-keyring-linter".into()));
-        }
-    } else {
+
         if ! args.quiet {
             err!(certs_invalid,
                  "  {} {} invalid and {} not linted.",
@@ -880,7 +886,11 @@ fn real_main() -> Result<()> {
 
         if args.fix {
             if unfixed_issue == 0 {
-                exit(0);
+                if bad_input {
+                    exit(1);
+                } else {
+                    exit(0);
+                }
             } else {
                 if ! args.quiet {
                     err!(unfixed_issue,
@@ -895,5 +905,9 @@ fn real_main() -> Result<()> {
         }
     }
 
+    if bad_input {
+        exit(1);
+    }
+
     Ok(())
 }
